!******************************************************************************!
!                                                                              !
!    class_loop.f90                                                            !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_loop.f90                                 |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.-N. Lang                             |            !
!        | `---' | UFO date: 04/2020                              |            !
!        |       | UFO version: 0.9                               |            !
!        |       | Generation date: 2020-03-10 10:59             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_loop
  use constants_mdl
  use input_mdl, only: error_mdl
  use lcSSV, only: loopSSV
  use lcVVV, only: loopVVV
  use lcVVVV, only: loopVVVV
  use lcFFVa, only: loopFFVa
  use lcSSVV, only: loopSSVV
  implicit none

  interface compute_loop_mdl
    module procedure computeLoop2, computeLoop3, computeLoop4, computeLoop5,   &
    computeLoop6, computeLoop7, computeLoop8
  end interface 

  contains

  subroutine computeLoop2(ty, co, wl, P1, PL1, m, wp, riMaxIn, riMaxOut,       &
  rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop2

  subroutine computeLoop3(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,    &
  riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(SSV0, VSS0, SVS0)
      call loopSSV(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(VVV0)
      call loopVVV(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(FVFa0, FVFa1, FFaV0, FFaV1, FaFV0, FaFV1, FaVF0, FaVF1, VFFa0, VFFa1, &
    VFaF0, VFaF1)
      call loopFFVa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop3

  subroutine computeLoop4(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp &
  , riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(VVVV0)
      call loopVVVV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,      &
      riMaxIn, riMaxOut, rankInc)
    case(SVVS0, SVSV0, SSVV0, VSSV0, VSVS0, VVSS0)
      call loopSSVV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,      &
      riMaxIn, riMaxOut, rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop4

  subroutine computeLoop5(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop5

  subroutine computeLoop6(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop6

  subroutine computeLoop7(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop7

  subroutine computeLoop8(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, w7, P7, PL7, m, wp, riMaxIn, riMaxOut,      &
  rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (kind = dp), intent(in), dimension(0:3) :: P7, PL7
    complex (kind = dp), intent(in), dimension(0:3) :: w7
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop8

end module class_loop
