!******************************************************************************!
!                                                                              !
!    fill_ctparameters.f90                                                     !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   fill_ctparameters.f90                          |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.-N. Lang                             |            !
!        | `---' | UFO date: 04/2020                              |            !
!        |       | UFO version: 0.9                               |            !
!        |       | Generation date: 2020-03-10 10:59             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module fill_ctparameters
  use class_ctparameters
  use modelintegrals, only: set_modelintegrals
  contains

  subroutine set_renoscheme_mdl(ctparam,renoscheme)
    use input_mdl, only: warning_mdl
    character(*), intent(in) :: ctparam
    character(*), intent(in) :: renoscheme

    select case(ctparam)
    case default
      call warning_mdl('No renoscheme available for '// trim(ctparam) // '.',  &
      where='set_renoscheme_mdl')
    end select 
  end subroutine set_renoscheme_mdl

  function get_renoscheme_mdl(ctparam)
    use input_mdl, only: warning_mdl
    character(*), intent(in) :: ctparam
    character(len=100) :: get_renoscheme_mdl

    select case(ctparam)
    case default
      call warning_mdl('No renoschemes for '// trim(ctparam) // '.', where     &
      ='get_renoscheme_mdl')
      get_renoscheme_mdl = "NONE"
    end select 
  end function get_renoscheme_mdl

  subroutine print_renoschemes_mdl()
    use input_mdl, only: nx
    character(len=100) :: active_renoscheme
    character(len=14) :: fmt = '(4x,a15,a30)'
  end subroutine print_renoschemes_mdl

  subroutine print_counterterms_mdl()
    use input_mdl, only: nx
    character(len=26) :: fmtc = '(4x,a17,es23.16,x,es23.16)'
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Counterterm parameters'
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Particle counterterm parameter:'
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Coupling counterterm parameter:'
  end subroutine print_counterterms_mdl

  subroutine set_light_particles_mdl()
    use class_particles
    use input_mdl, only: nx, print_regularization_scheme
    if (print_regularization_scheme) then
      write(nx,'(1x,75("-"))')
      write(nx,*) 'Light fermions renormalization scheme choice:'
    end if 
    if (print_regularization_scheme) write(nx,'(1x,75("-"))')
  end subroutine set_light_particles_mdl

  subroutine init_CTparameters_mdl(reset_collier)
    logical, intent(in) :: reset_collier
    call set_light_particles_mdl
    call set_modelintegrals(reset_collier)
  end subroutine init_CTparameters_mdl
end module fill_ctparameters
